
#include <avr/io.h>
#include <util/delay.h>
#include "main.h"
#include "touch.h"


//Daten fr die Umrechnung
#define KENN_VALNUM 10
uint16_t raKenn_adc[KENN_VALNUM] = {0, 26, 52, 78, 109, 135, 167, 193, 219, 255};
uint16_t raKenn_tmp[KENN_VALNUM] = {0, 2, 10, 27, 61, 105, 174, 256, 348, 511};


void pause_ms(unsigned int  ms) {
	unsigned int zaehler;
	
	while (ms) {
		zaehler = F_CPU / 5000;
		
		while (zaehler) {
			asm volatile ("nop");
			zaehler--;
		}
		ms--;
	}
}

uint16_t GetPwm(uint16_t val) {
	uint8_t i;
	//ermitteln, in welchem Bereich sich der Wert befindet. Es wird solange gesucht, bis der
	//Wert grer als der Wert in der Kennwert-Tabelle ist. diese Position wird in i erhalten und man
	//kann damit prima rechnen ;)
	for(i = 1; i < KENN_VALNUM; i++) {
		if(raKenn_adc[i] >= val) {
			break;
		}
	}

	i--;
	uint16_t adc_d = raKenn_adc[i+1] - raKenn_adc[i];
	uint16_t tmp_d = raKenn_tmp[i+1] - raKenn_tmp[i];
	tmp_d *= 255; //Genauigkeit bisschen hochschrauben
	val -= raKenn_adc[i];
	return ((tmp_d/adc_d)*val)/255 + raKenn_tmp[i];
}


void SetPWM(uint8_t channel, uint8_t value) {
	
	//Ermitteln, welche Register angefasst werden mssen.
	uint8_t com01 = (channel == 0) ? (1<<COM0A1) : (1<<COM0B1);
	uint8_t pin = (channel == 0) ? (1<<PB0) : (1<<PB1);
	
	
	if(value == 0) {
		//Wenn der Wert 0 ist, muss der PWM deaktiviert werden, sonst leuchtet die LED noch ein bisschen
		TCCR0A &= ~com01;
		PORTB &= ~pin;
	} else {
		//PWM aktivieren
		TCCR0A |= com01;
		//Etwas unschn, aber 0 und 1 im Register ist ein Unterschied, den man sieht
		//zwischen 254 und 255 sieht man den Unterschied nicht mehr.
		if(value < 255) value--; //hmpf
		if(channel == 0) {
			OCR0A = value;
		} else {
			OCR0B = value;
		}
	}
}


int main(void) {
  uint8_t i;
	LEDDDR |= (1<<LED0) | (1<<LED1);

	//Timer Initialisieren
	TCCR0A=(1<<COM0A1)|(1<<COM0B1)|(1<<WGM00)|(1<<WGM01);
	TCCR0B=(1<<CS01);

	//Interrupts initialisieren
	GIMSK=0x00;
	MCUCR=0x00;


	uint8_t brightness = 0;
	uint8_t fade = FADE_OFF;
	uint16_t b, ba, bb;
	while(1) {
		//Ersten Sensor (UP) messen
		i = getcap2(1<<KEY0);
		if(i < 25) {
			//wird stark gedrckt -> automatisch dimmen
			fade = FADE_UP;
		} else if(i < 40) {
			//Bei leichten Druck nur solange, bis losgelassen wird und automatisches Dimmen deaktivieren
			if(brightness < 255) brightness++;
			if(fade == FADE_DOWN) fade = FADE_OFF;
		}

		//zweiten Sensor (DOWN) messen
		i = getcap2(1<<KEY1);
		if(i < 25) {
			fade = FADE_DOWN;
		} else if(i < 40) { 
			if(brightness > 0) brightness--;
			if(fade == FADE_UP) fade = FADE_OFF;
		}
		
		//Automatisches Dimmen
		if(fade == FADE_DOWN) {
			if(brightness == 0) { 
				//Zielwert erreicht -> dimmen beenden
				fade = FADE_OFF; 
			} else {
				 brightness--; 
			}
		} else if(fade == FADE_UP) {
			if(brightness == 255) {
				fade = FADE_OFF; 
			} else {
				brightness++; 
			}
			//Das hochdimmen erfolgt langsamer, sieht besser aus
			pause_ms(5);
		}

		//Kennlinie abarbeiten
		b = GetPwm((uint16_t)brightness);
		ba = b / 2; //Komponente A der Helligkeit
		bb = b - ba; //Komponente B der Helligkeit

		SetPWM(0, (uint8_t)ba);
		SetPWM(1, (uint8_t)bb);
		pause_ms(5); //Alles mit etwas Gemtlichkeit...

  }
}





